#!/usr/bin/perl -w
# $Header: /home/hiro/projects/tv/RCS/dl-iepg.pl,v 1.11 2011/10/25 18:37:48 hiro Exp $
#
# dl-iepg.pl
#
#	TV program iEPG downloader run daily by cron
#
#	(c)2009 Hiro Sugawara
#	Distributed under terms of GPL 2.0
#

use strict;
use POSIX;
use File::Basename;
use Sys::Syslog;

my $myname	= basename $0;
my $cwd		= `pwd`;
chomp $cwd;
my $mydir	= dirname $0;
if ($mydir eq '.') {
	$mydir = '';
}
unless ($mydir =~ /^\//) {
	$mydir	= "$cwd/$mydir";
}
$mydir	=~ s/\/$//;
my $cfile	= "$mydir/config/tv.conf";

openlog("$myname\[$$\]", "", "local0");

sub help {
	print <<_END;
Usage: $myname [-c config] {-r | goo | ri | sn | tv | tt}
    -c config      configuration file ($cfile)
    -r             use a randomly chosen TV program site
    goo            goo (not recommended)
    ri             Rakuten-Infoseek
    sn             So-net
    tv             TV Guide
    tt             The Television
_END
}

# Forward declaration of config parameters
our $VIDEOROOTDIR;
our $CONFIGROOTDIR;
our $CHANNELMAP;
our $iCHANNELMAP;
our $CHANNELAREA;

my $tpsite;
#---------------------------------------------------------------
# Parse options
#---------------------------------------------------------------
while ($#ARGV >= 0 && $ARGV[0] =~ /^-/) {
	my $opt	= shift @ARGV;
	if ($opt eq '-c') {
		$cfile	= shift @ARGV;
	} elsif ($opt eq '-r') {
		$tpsite	= 'random';
	} elsif ($opt eq '-h' || $opt eq '-?') {
		help();
		exit 0;
	} else {
		die "Unknown option $opt\n";
	}
}
unless (defined($tpsite)) {
	$tpsite	= shift @ARGV;
}

require "$mydir/6ga-common.pl";

#-------------------------------------------------------------
# Check options
#-------------------------------------------------------------
defined($VIDEOROOTDIR)	|| die "VIDEOROOTDIR undefined";
chdir "$VIDEOROOTDIR/iepg"		|| die "can't chdir to $VIDEOROOTDIR/iepg";

#-------------------------------------------------------------
# Validate and select TV program site
#-------------------------------------------------------------
my %tps	= (
	##'goo'	=> \&goo_download,
	##'ri'	=> \&ri_download,
	##'sn'	=> \&sn_download,
	##'tv'	=> \&tv_download,
	##'tt'	=> \&tt_download,
	'epg'	=> \&epgrec_download,
);

my $downloader;
if (!defined($tpsite)) {
	help();
	exit 1;
} elsif ($tpsite eq 'random') {
	srand(time);
	my @keys	= keys(%tps);
	my $key;
	do {
		$key	= $keys[rand($#keys + 1)];
	} until ($key ne 'goo');
	$downloader	= $tps{$key};
} else {
	$downloader	= $tps{$tpsite};
}
unless (defined($downloader)) {
	help();
	exit 1;
}

#-------------------------------------------------------------
# Run exclusively
#-------------------------------------------------------------
my $lockfile	= $ENV{HOME} . "/lock/$myname";
$lockfile	=~ s/\.[^.]*$/.lock/;

umask 0377;
until (open LOCKFILE, ">$lockfile") {
	open(LOCKFILE, $lockfile) || die "can't open lockfile $lockfile for read\n";
	my $pid = <LOCKFILE>;
	close LOCKFILE;
	if (defined($pid)) {
		chomp $pid;
		if (-d "/proc/$pid") {
			exit 0;
		}
	}
	unlink $lockfile	|| die "can't delete stale lockfile $lockfile";
}
print LOCKFILE "$$\n";
close LOCKFILE;

logit("Starting");

delete_stalefiles();

#-------------------------------------------------------------
# Loop for all downloadable iEPGs
#-------------------------------------------------------------
my $WGET	= 'wget -q';
my $DATESPAN	= 8;
my $AREA	= defined($CHANNELAREA)	? $CHANNELAREA	: '008';
&$downloader();

logit("Downloading iEPG completed");
unlink $lockfile;
0;

#============================================================
# Goo
#============================================================
sub goo_download {
	my $BASEURL	= 'http://tv.goo.ne.jp';
	my @TEMPURLS	= (
		$BASEURL . "/contents/epg/$AREA/VHF_1/%s_0024/index.html",
		$BASEURL . "/contents/epg/$AREA/UHF-BSanalog_1/%s_0024/index.html");

	my $now	= time;
	my $datespan	= $DATESPAN;

	umask 022;
	while ($datespan-- > 0) {
		foreach my $TEMPURL (@TEMPURLS) {
			my $url	= sprintf $TEMPURL, strftime("%Y%m%d", localtime($now));
			logit("Parsing '$url'");
			open DISP, "$WGET -O - '$url'|"
						|| logdie("can\'t open wget -O - '$url'");
			while (<DISP>) {
				chomp;
				my $line	= $_;
				while ($line	=~
/href=["'](\/contents\/program\/$AREA\/\d{4}\/\d{8}_\d{4}\/iepg\.tvpi)["']/) {
					my $path	= $1;
					my $iepgurl	= "$BASEURL$path";
					$path =~ /\/(\d{4})\/(\d{8})_(\d{4})\//;
					my $id		= $1;
					my $datetime	= "$2$3";
					my $fname	= sprintf("%s%s.iepg", $datetime, $id);

					iEPGfname($fname, $iepgurl);
					$line	=~ s/^.*$path//;
				}
			}
			close DISP;
		}
		$now	+= 3600 * 24;	# Advance date
	}
}

#============================================================
# Rakuten/Infoseek
#============================================================
sub ri_download {
	my $BASEURL	= 'http://tv.infoseek.co.jp';
	my $PORTALURL	= $BASEURL . '/?lhour=24';
	my $TEMPURL	= $PORTALURL . '&sdate=%s';

	my $datespan	= $DATESPAN;
	my $now	= time;

	umask 022;
	while ($datespan-- > 0) {
		my $url	= sprintf $TEMPURL, strftime("%Y%m%d", localtime($now));
		logit("Parsing '$url'");
		open DISP, "$WGET -O - '$url'|"
					|| logdie("can\'t open wget -O - '$url'");
		while (<DISP>) {
			chomp;
			if (
/href=["'](\/tvpi\.epg\?pg=tv_record\.html&program=p\d{4})(\d{12})(&area=$AREA)["']/) {
				my $id		= $1;
				my $datetime	= "$2";
				my $trailer	= $3;
				my $iepgurl	= "$BASEURL$id$datetime$trailer";
				my $fname	= sprintf("20%s%s.iepg",
									substr($datetime, 0, 10),
									substr($id, -4));

				iEPGfname($fname, $iepgurl);
			}
		}
		close DISP;
		$now	+= 3600 * 24;	# Advance date
	}
}

#============================================================
# The Television
#============================================================
sub tt_download {
	my $BASEURL	= 'http://www.television.co.jp/programlist';
	my $PORTALURL	= $BASEURL . '/guide.php?range=24';
	my $TEMPURL	= $PORTALURL . '&day=%d';

	my $datespan	= $DATESPAN;
	my $now	= time;
	my $day = 0;

	umask 022;
	while ($datespan-- > 0) {
		my $url	= sprintf($TEMPURL, $day++);
		logit("Parsing '$url'");
		open DISP, "$WGET -O - '$url'|"
					|| logdie("can\'t open wget -O - '$url'");
		while (<DISP>) {
			chomp;
			if (
		/href=["'](iepg.php\?id=\d{10}-\d{1,3}-\d{1,4}-)(\d{1,10})["']/) {
				my $id		= $1;
				my $ut		= $2;
				my $iepgurl	= "$BASEURL/$id$ut";
				my $fname	= sprintf("%s.iepg",
									strftime("%Y%m%d%H%M", localtime($ut)));

				iEPGfname($fname, $iepgurl);
			}
		}
		close DISP;
		$now	+= 3600 * 24;	# Advance date
	}
}

#============================================================
# TV-Guide
#============================================================
sub tv_download {
	my $BASEURL	= 'http://www.tvguide.or.jp';
	my $PORTALURL	= $BASEURL . '/TF1101LS.php?time=24';
	my $TEMPURL	= $PORTALURL . '&date=%s';

	my $PATTERN	= '["\']TF0001SY\.php\?programId=(\d*)&broadcastingDay=(\d{4})-(\d{1,2})-(\d{1,2})&packNo=\d*["\']';

	my $datespan	= $DATESPAN;
	my $now	= time;

	umask 022;
	while ($datespan-- > 0) {
		my $url	= sprintf $TEMPURL, strftime("%Y%m%d", localtime($now));
		logit("Parsing '$url'");
		open DISP, "$WGET -O - '$url'|"
					|| logdie("can\'t open wget -O - '$url'");
		while (<DISP>) {
			chomp;
			if (/href=$PATTERN/	|| /onclick="javascript:popup\($PATTERN\);/) {
				my $id	= $1;
				my $yr	= $2;
				my $mo	= $3;
				my $da	= $4;
				my $iepgurl
					= "$BASEURL/TF0706ET.php?programId=$id&date=$yr$mo$da";
				my $fname	= sprintf("%s%02d%02d$id.iepg", $yr, $mo, $da);

				iEPGfname($fname, $iepgurl);
			}
		}
		close DISP;
		$now	+= 3600 * 24;	# Advance date
	}
}

#============================================================
# So-net
#============================================================
sub sn_download {
	my $BASEURL	= 'http://tv.so-net.ne.jp';
	my $PORTALURL	= $BASEURL . '/chart/23.action?span=24';
	my $TEMPURL	= $PORTALURL . '&head=%s';

	my $datespan	= $DATESPAN;
	my $now	= time;

	umask 022;
	while ($datespan-- > 0) {
		my $url	= sprintf $TEMPURL, strftime("%Y%m%d%H%M", localtime($now));
		logit("Parsing '$url'");
		open DISP, "$WGET -O - '$url'|"
					|| logdie("can\'t open wget -O - '$url'");
		while (<DISP>) {
			chomp;
			if (/href=["'](\/iepg\.tvpi\?id=)(\d{18})["']/) {
				my $id		= $1;
				my $datetime	= $2;
				my $iepgurl	= "$BASEURL$id$datetime";
				my $fname	= sprintf("%s.iepg", substr($datetime, 6, 12));

				iEPGfname($fname, $iepgurl);
			}
		}
		close DISP;
		$now	+= 3600 * 24;	# Advance date
	}
}

#=======================================================
# Rename iEPFfname
#=======================================================
sub iEPGfname {
	my ($fname, $iepgurl, $iepgref)	= @_;
	if (defined($iepgurl)) {
		my %iepg;
		system "$WGET -O $fname '$iepgurl' | sed 's|^\[..\]||g'";
		unless (readiEPG($fname, \%iepg) && defined($iepg{'station'})) {
			logit("Can't find station in iEPG $fname ($iepgurl)");
			unlink $fname;
			return;
		}
		$iepgref = \%iepg;
	} elsif (!defined($iepgref)) {
		logit("iepgref undefined");
		return;
	}
	my $channel	= findChannel($iepgref->{'station'});
	if (defined($channel)) {
		my $fname2	= sprintf("%04d%02d%02d%02d%02d-%03d.iepg",
							$iepgref->{year},
							$iepgref->{month},
							$iepgref->{date},
							substr($iepgref->{start}, 0, 2),
							substr($iepgref->{start}, 3, 2),
							$channel);
		if (defined($fname)) {
			rename $fname, $fname2;
		} else {
			umask 0133;
			unlink $fname2;
			open IEPG, ">$fname2" || die;
			foreach my $key (keys(%$iepgref)) {
				if (defined($iepgref->{$key})) {
					printf IEPG "%s: %s\n", $key, $iepgref->{$key};
				}
			}
			close IEPG;
		}

		if (exists($iepgref->{'d-channel'})) {
			my $fname3	= sprintf("%04d%02d%02d%02d%02d-%s.iepg",
							$iepgref->{year},
							$iepgref->{month},
							$iepgref->{date},
							substr($iepgref->{start}, 0, 2),
							substr($iepgref->{start}, 3, 2),
							$iepgref->{'d-channel'});
			symlink $fname2, $fname3;
		}
	} else {
		# Reception unavailable
		unlink $fname	if defined($fname);
	}
	if ($iepgurl) {
		sleep 2 + rand(5);
	}
}


#=======================================================
# Search ichannel map for local channel
#=======================================================
my %iChannels;
sub iChannel2local {
	my ($station)	= @_;
	my $channel	= $iChannels{$station};
	unless ($channel) {
		$iChannels{$station}	= findiChannel($station);
		$channel	= $iChannels{$station};
	}
	return $channel;
}

#=======================================================
# Look up iEPG channel for channel number
#=======================================================
sub findiChannel {
	my ($station)	= @_;
	my $channel	= 0;

	open(CMAP, "$CONFIGROOTDIR/channels/$iCHANNELMAP")
							|| die "can't open $iCHANNELMAP";
	while (<CMAP>) {
		chomp;
		s/#.*$//;
		last if (/\[$station\]/);
	}

	while (<CMAP>) {
		chomp;
		s/#.*$//;
		if (/^\s*channel\s*=\s*(\d+)/) {
			$channel = $1;
			last;
		}
	}
	close CMAP;
	return $channel;
}

#============================================================
# Look up station name for channel number
#============================================================
my %channels;
sub findChannel {
	my ($station)	= @_;
	my $channel	= $channels{$station};

	return $channel	if ($channel);
	open(CMAP, "$CONFIGROOTDIR/channels/$CHANNELMAP")
							|| die "can't open $CHANNELMAP";
	while (<CMAP>) {
		chomp;
		s/#.*$//;
		last if (/\[$station\]/);
	}

	while (<CMAP>) {
		chomp;
		s/#.*$//;
		if (/^\s*channel\s*=\s*(\d+)/) {
			$channel = $1;
			$channels{$station}	= $channel;
			last;
		}
	}
	close CMAP;
	return $channel;
}

#============================================================
# Delete stale iEPF files
#============================================================
sub delete_stalefiles{
	my $today	= strftime "%Y%m%d", localtime(time - 86400 * 7);
	if (opendir DIR, '.') {
		while (my $file = readdir DIR) {
			if ((-f $file || -l $file)
				&& $file =~ /\.iepg$/
				&& $file lt $today) {
				unlink $file;
			}
		}
		closedir DIR;
	}
}

#============================================================
# Log and die
#============================================================
sub logdie {
	my ($fmt, @args)	= @_;
	syslog('info', "$fmt", @args);
	die sprintf($fmt, @args);
}

#============================================================
# Log information
#============================================================
sub logit {
	my ($fmt, @args)	= @_;
	syslog('info', "$fmt", @args);
}

##
## From local EPGREC
##

use DBI();
use Text::Iconv;
sub epgrec_download {
	my %DtoAchlist = (
		27	=> 1,
		26	=> 3,
		25	=> 4,
		22	=> 6,
		21	=> 8,
		24	=> 10,
		23	=> 12,
		28	=> 16,
	);
	my $converter = Text::Iconv->new('UTF-8', 'SHIFT-JIS');
	my $dbh = DBI->connect('DBI:mysql:database=epgrec;host=localhost',
	                         'dvr2', 'os9000', {'RaiseError' => 1});
	$dbh->do('set names UTF8');
	my $sth = $dbh->prepare('SELECT * FROM Recorder_programTbl');
	$sth->execute();
	while (my $ref = $sth->fetchrow_hashref()) {
		my $channel	= $ref->{'channel'};
		my $type	= $ref->{'type'};
		my $title	= $ref->{'title'};
		my $description	= $ref->{'description'};
		my $starttime	= $ref->{'starttime'};
		my $endtime	= $ref->{'endtime'};
		my $id	= $ref->{'id'};

		my ($year, $month, $date)	= split /[\- :]/, $starttime;
		my $start	= substr((split / /, $starttime)[1], 0, 5);
		my $end		= substr((split / /, $endtime)[1], 0, 5);
		my $program_title	= $converter->convert($title);
		if (defined($program_title)) {
			# Remove [x] icons
			$program_title	=~ s/\x{81}\x{79}..\x{81}\x{7a}//g;
		}
		my $program_subtitle	= $converter->convert($description);

		my $ch = $DtoAchlist{$channel};
		defined($ch)	|| next;
		
		open(CMAP, "$CONFIGROOTDIR/channels/$CHANNELMAP") || next;
		my $sname;
		my $station;
		while (<CMAP>) {
			chomp;
			if (/channel[[:space:]]*=[[:space:]]*$ch\b/) {
				$station = $sname;
				$station =~ s/[[:space:]]*\[//;
				$station =~ s/\][[:space:]]*//;
				last;
			}
			$sname = $_	if ($_);
		}
		close CMAP;
		next unless defined($station);

		my %iepg = (
		'Content-type'	=> 'application/x-tv-program-info; charset=shift_jis',
		'version'	=> '1',
		'station'	=> $station,
		'year'	=> $year,
		'month'	=> $month,
		'date'	=> $date,
		'start'	=> $start,
		'end'	=> $end,
		'program-title'	=> $program_title,
		'program-subtitle' => $program_subtitle,
		'chennel'	=> $ch,
		'd-channel'	=> "$type$channel",
		'program_id'	=> $id,
		);
		iEPGfname(undef, undef, \%iepg);
	}
	$sth->finish();
	$dbh->disconnect();
}
